// repeat-CNT value
`include "CNT_file"

//----------------------------
// free-running clk oscillator
//----------------------------
`define CYCLE 10
module clkgen (output logic clk);
  initial begin
    clk <= '0;
    forever #(`CYCLE/2) clk = ~clk;
  end
endmodule

module top;
  logic [31:0] x;
  logic [31:0] data, m;
  logic        setval;
  logic        clk;
  int          cnt;
  int          max = `CNT-5;

  clkgen      cg   (.clk(clk));

  `ifdef SLOW
  A1 i1 (.x(x), .data(data), .m(m), .setval(setval), .clk(clk));
  `else
  A2 i1 (.x(x), .data(data), .m(m), .setval(setval), .clk(clk));
  `endif

  initial begin
    data   <= $urandom;
    m      <= $urandom;
    setval <= '1;
    display;
    cnt++;
    forever @(posedge clk) begin
      data   <= $urandom;
      m      <= $urandom;
      setval <= ~setval;
      if ((cnt <5) || (cnt>max)) display;
      cnt++;
    end
  end

  //-------------------------------------------------------------
  // Sanity check to make sure the test is running:
  // Display first 5 and last 5 x, data, m, setval and cnt values
  //-------------------------------------------------------------
  function void display;
    $strobe("x=%8h  data=%8h  m=%8h  setval=%b  cnt=%d", 
             x,     data,     m,     setval,    cnt);
  endfunction

  initial begin
    repeat(`CNT) @(posedge clk);
    $finish;
  end
endmodule

// Figure 6:  Example of reducing garbage collection overhead with object management 
