`include "CYCLE.sv"
`define Tdrive #(0.2*`CYCLE)

`include "CNT_file"

module top;
  logic [31:0] c, cout;
  logic [31:0] a, b, ain, bin;
  logic        clk;

  int          data = 3; // Random-3 initialization

  int          cnt;
  int          max = `CNT-5;

  clkgen      cg   (.clk(clk));

  clocking cb1 @(posedge clk);
    default input #1step output `Tdrive;
    input  c;
    output a, b;
  endclocking

  BENM8 i1 (.c(c), .a(a), .b(b), .clk(clk));

  initial begin // Stimulus
    initialize();
    forever drive_inputs();
  end

  initial begin // Conditional output sampling
    @cb1;
    sample_ports();
    //-------------------------------------------------------------
    // Sanity check to make sure the test is running:
    // Display first and last sampled values
    //-------------------------------------------------------------
    forever begin
      sample_ports();
      if ((cnt <5) || (cnt>max)) display();
      cnt++;
    end
  end

  initial begin // Clock cycle counter - then $finish
    repeat(`CNT) @(posedge clk);
    $finish;
  end

  task initialize(); // initialize inputs at time-0
    a <= '0;
    b <= '0;
  endtask

  task drive_inputs(); // drive inputs %20 of CYCLE after posedge clk
    @cb1;
    //cb1.a <= $urandom;
    //cb1.b <= $urandom;
    cb1.a <= data;
    cb1.b <= ~data;
    data += 3;
  endtask

  //------------------------------------------------------
  // sample ports is synchronized to posedge clk
  // inputs are sample on posedge clk
  // outputs are sampled #1step before next posedge clk
  // task is done on next posedge clk - hence synchornized
  //     to next posedge clk
  //------------------------------------------------------
  task sample_ports();
    ain = a;
    bin = b;
    @cb1;
    cout = cb1.c;
  endtask

  function void display();
    $display("c=%8h  a=%8h  b=%8h  cnt=%d", 
              cout, ain, bin, cnt);
  endfunction
endmodule

// Figure 6:  Example of reducing garbage collection overhead with object management 
