// repeat-CNT value
`include "CNT_file"

//----------------------------
// free-running clk oscillator
//----------------------------
`include "CYCLE.sv"
module clkgen (output logic clk);
  initial begin
    clk <= '0;
    forever #(`CYCLE/2) clk = ~clk;
  end
endmodule

module top;
  logic  [3:0] loop = 4'h1; // set low bit
  logic        txactive, active_port;
  logic        clk;

  int          cnt;
  int          max = `CNT-5;

  clkgen      cg   (.clk(clk));

  clocking cb1 @(posedge clk);
    default input #1step output `Tdrive;
    output txactive, loop;
  endclocking

  `ifdef B6A

  BENM9A i1 (.txactive(txactive), .clk(clk));

  `endif

  `ifdef B6B

  BENM9B i1 (.txactive(txactive), .clk(clk));

  `endif

  `ifdef B6C

  BENM9C i1 (.txactive(txactive), .clk(clk));

  `endif

  initial begin // Stimulus
    $timeformat(-9,0,"ns",10);
    $display("%t: CNT=%0d   max=%0d", $time, `CNT, max);
    initialize();
    forever drive_inputs();
  end

  initial begin // Conditional output sampling
    @cb1;
    sample_ports();
    //-------------------------------------------------------------
    // Sanity check to make sure the test is running:
    // Display first and last values
    //-------------------------------------------------------------
    forever begin
      if ((cnt <5) || (cnt>max)) sample_ports();
      else @cb1 cnt++;
    end
  end

  initial begin // Clock cycle counter - then $finish
    repeat(`CNT) @(posedge clk);
    $finish;
  end

  task initialize(); // initialize inputs at time-0
    txactive <= '1;
  endtask

  task drive_inputs(); // drive inputs %20 of CYCLE after posedge clk
    @cb1;
    cb1.txactive <= loop[0];
    cb1.loop     <= {loop[2:0], loop[3]}; // rotate loop bit
  endtask

  //------------------------------------------------------
  // sample ports is synchronized to posedge clk
  // inputs are sample on posedge clk
  // outputs are sampled #1step before next posedge clk
  // task is done on next posedge clk - hence synchornized
  //     to next posedge clk
  //------------------------------------------------------
  task sample_ports();
    //$display("Starting sample_ports");
    active_port = txactive;
    @cb1;
    cnt++;
    display();
  endtask

  function void display();
    $display("%t: txactive=%b  cnt=%d", 
              $time, active_port, cnt);
  endfunction
endmodule

// Figure 6:  Example of reducing garbage collection overhead with object management 
