module BEN11A (
  output logic [31:0] counter,
  input  logic        req, ack,
  input  logic        clk, rst_n);

  enum logic [2:0] {
                    IDLE,
                    S1,
                    S2,
                    S3,
                    S4,
                    XXX} state;

  // Sequentially executed code
  // While there is no req, stay in IDLE state
  // While there is no ack, stay in S1 state
  // When ack become active, increment counter value (S2 state)
  //     If req is still high, goto S3 else goto S4 and wait for !ack
  // While req is still high, stay in S3 state
  // While ack is still high, stay in S4 state
  // When done, start over in IDLE state

  initial @(negedge rst_n) begin
    forever begin: fsm
      state   = IDLE;
      counter = '0;
      @(posedge rst_n);
      @(posedge clk); // Stay in IDLE until the next clk

      forever begin
        while (!req) @(posedge clk); // Stay in IDLE
                                state = S1;
                     @(posedge clk); // S1
        while (!ack) @(posedge clk); // Stay in S1
                                state = S2;
          counter = counter + 1;     // S2
                     @(posedge clk); // S2 for just one clk
        if (req) begin
	                        state = S3;
                     @(posedge clk); // S3
          while (req)@(posedge clk); // Stay in S3
	end
	                        state = S4;
	             @(posedge clk); // S4
        while ( ack) @(posedge clk); // Stay in S4
                                state = IDLE;
                     @(posedge clk); // IDLE
      end
    end
  end
  
  always @(negedge rst_n) begin
    disable fsm;
  end
endmodule
