//----------------------------------------------------------------------
// This File: tb_agent.sv
//
// DVCon 2019 Paper Example
//
// Sunburst Design (Provo, UT):
//            cliffc@sunburst-design.com
//            www.sunburst-design.com
//----------------------------------------------------------------------

class tb_agent extends uvm_agent;
  `uvm_component_utils(tb_agent)
                                                                                
  virtual dut_if vif;

  uvm_analysis_port #(trans1) ap;
  tb_driver     drv;
  tb_monitor    mon;
  tb_sequencer  sqr;

  function new (string name, uvm_component parent);
    super.new(name, parent);
  endfunction

  virtual function void build_phase(uvm_phase phase);
    super.build_phase(phase);
    drv =    tb_driver::type_id::create("drv", this);
    sqr = tb_sequencer::type_id::create("sqr", this);
    mon =   tb_monitor::type_id::create("mon", this);
    ap  = new("ap", this);
    get_vif();
  endfunction

  virtual function void connect_phase(uvm_phase phase);
    super.connect_phase(phase);
    drv.seq_item_port.connect(sqr.seq_item_export);
    drv.vif = vif;
    mon.vif = vif;
    mon.ap.connect(ap);
  endfunction

  // Only works for UVM (uses uvm_config_db)
  function void get_vif;
    if(!uvm_config_db#(virtual dut_if)::get(this,"","vif",vif))
        `uvm_fatal("NOVIF",{"virtual interface must be set for:",
                             get_full_name(),".vif"});
  endfunction
endclass
